﻿using Dynamicweb.Configuration;
using Dynamicweb.Core;
using Dynamicweb.Ecommerce.LiveIntegration.Logging;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using System.Xml;

namespace Dynamicweb.Ecommerce.LiveIntegration.Licensing
{
    internal class LicenseService
    {
        private static ConcurrentDictionary<string, LicenseStatus> EndpointCollection = new ConcurrentDictionary<string, LicenseStatus>();
        private static Version MinVersion = new Version(1, 0, 0, 0);
        private static readonly string LiveIntegrationSettingsKey = "/Globalsettings/Modules/LiveIntegration";
        private static bool IsFeatureEnabled
        {
            get
            {
                return Security.Licensing.LicenseManager.LicenseHasFeature("eCom_DataIntegrationERPLiveIntegrationV2");
            }
        }

        internal static bool IsLicenseValid(string endpoint)
        {
            LicenseStatus status = GetStatus(endpoint);
            if (status != null && !status.Expired)
            {
                return status.Allowed;
            }
            else
            {
                return true;
            }
        }

        private static LicenseStatus GetStatus(string endpoint)
        {
            LicenseStatus status = null;
            EndpointCollection.TryGetValue(endpoint, out status);
            return status;
        }

        internal static void ValidateLicense(string endpoint, string response)
        {
            var status = GetStatus(endpoint);
            if (status == null || status.Expired)
            {
                XmlDocument doc;
                if (!Helpers.ParseResponseToXml(response, out doc))
                {
                    Logger.Instance.Log(ErrorLevel.DebugInfo, $"License Response is not valid XML");
                }
                else
                {
                    ValidateLicense(doc, endpoint, status);
                }
            }
        }

        internal static bool ValidateLicense(string endpoint, XmlDocument doc)
        {
            var status = GetStatus(endpoint);
            if (status == null || status.Expired)
            {
                ValidateLicense(doc, endpoint, status);
            }
            return IsLicenseValid(endpoint);
        }

        private static void ValidateLicense(XmlDocument response, string endpoint, LicenseStatus status)
        {
            bool updated = false;
            bool isNew = false;
            if (status == null)
            {
                status = new LicenseStatus();
                EndpointCollection.TryAdd(endpoint, status);
                isNew = true;
            }

            var versionAttribute = response.DocumentElement?.Attributes["version"];
            string versionValue = versionAttribute?.Value;
            if (isNew)
            {
                status.UpdateVersion(versionValue);
                ValidateLicense(status, versionAttribute);
            }
            else
            {
                if (status.IsDifferentVersion(versionValue))
                {
                    status.UpdateVersion(versionValue);
                    ValidateLicense(status, versionAttribute);
                    updated = true;
                }
                else
                {
                    if (!string.IsNullOrEmpty(versionValue))
                    {
                        //If it is the same version only check if HasFeature value was not changed
                        //If yes revalidate the license
                        if(status.IsFeatureEnabled.HasValue && status.IsFeatureEnabled.Value != IsFeatureEnabled)
                        {
                            ValidateLicense(status, versionAttribute);
                        }
                    }
                }
            }
            if (!string.IsNullOrEmpty(versionValue) && (isNew || updated))
            {
                SaveVersions();
            }
        }

        private static void ValidateLicense(LicenseStatus status, XmlAttribute versionAttribute)
        {
            if (versionAttribute != null)
            {
                bool isValid = IsVersionValid(versionAttribute.Value);
                bool isFeatureEnabled = true;
                if (!isValid)
                {
                    isFeatureEnabled = IsFeatureEnabled;
                }
                status.UpdateStatus(isValid);
                status.SetFeatureEnabled(isFeatureEnabled);
            }
            else
            {
                status.UpdateStatus(true);
            }
        }

        private static bool IsVersionValid(string version)
        {
            VersionInfo erpVersion;
            if (VersionInfo.TryParse(version, out erpVersion))
            {
                if (erpVersion.AppVersion >= MinVersion)
                {
                    if (IsFeatureEnabled)
                    {
                        return true;
                    }
                    else
                    {
                        Logger.Instance.Log(ErrorLevel.Error, $"License is not valid.");
                        return false;
                    }
                }
                else
                {
                    return true;
                }
            }
            else
            {
                return false;
            }
        }

        internal static void SaveSettings()
        {
            if(UrlHandler.Instance.GetWebServiceUrlsCount() > 0)
            {
                SaveVersions();
            }
            else
            {
                EndpointCollection.Clear();
            }
        }

        private static void SaveVersions()
        {
            string versions = string.Join("|",
                EndpointCollection.Values.Where(s => s != null && !string.IsNullOrEmpty(s.ResponseVersion)).Select(s => s.ResponseVersion).Distinct());
            var itemsToSave = new Dictionary<string, string>();
            itemsToSave.Add($"{LiveIntegrationSettingsKey}/Versions", versions);
            itemsToSave.Add($"{LiveIntegrationSettingsKey}/LastVersionTime", DateTime.Now.Ticks.ToString());
            SystemConfiguration.Instance.SetValue(itemsToSave);
        }
    }
}
