﻿using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.LiveIntegration.Extensions;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Ecommerce.Orders.SalesDiscounts;
using Dynamicweb.Environment;
using Dynamicweb.Extensibility.AddIns;
using System;
using System.Linq;
using System.Xml;

namespace Dynamicweb.Ecommerce.LiveIntegration
{
    /// <summary>
    /// Sales Discount Provider to handle discount lines coming from the ERP.
    /// </summary>
    /// <seealso cref="SalesDiscountProvider" />
    [AddInName("Live integration discount"),
    AddInDescription("Apply discounts retrieved via live integration")]
    public class LiveSalesDiscountProvider : SalesDiscountProvider
    {
        private static readonly string DiscountTranslationCultureSessionKey = "DiscountTranslationCulture";

        /// <summary>
        /// Gets or sets a value indicating whether to hide discount value group box in the UI.
        /// </summary>
        /// <value><c>true</c> if it is needed to hide discount value group box; otherwise, <c>false</c>.</value>
        public override bool HideDiscountValueGroupBox
        {
            get
            {
                return true;
            }

            set
            {
                base.HideDiscountValueGroupBox = true;
            }
        }

        /// <summary>
        /// Processes the order.
        /// </summary>
        /// <param name="order">The order.</param>
        /// <example>
        /// Look on example for <c>SalesDiscountProvider</c> class
        /// </example>
        public override void ProcessOrder(Order order)
        {
            var orderLines = GetDiscountOrderLinesFromCache(order);
            if (orderLines != null)
            {
                Diagnostics.ExecutionTable.Current.Add("LiveIntegration.LiveSalesDiscountProvider.ProcessOrder START");
                foreach (var orderLine in orderLines)
                {
                    if(!string.Equals(orderLine.Order.Id, order.Id, StringComparison.OrdinalIgnoreCase))
                    {
                        orderLine.Order = order;
                    }
                    if (!order.OrderLines.Contains(orderLine))
                    {
                        order.OrderLines.Add(orderLine.CloneOrderLine());
                    }
                }
                Diagnostics.ExecutionTable.Current.Add("LiveIntegration.LiveSalesDiscountProvider.ProcessOrder END");
            }
        }

        /// <summary>
        /// Adds to cache.
        /// </summary>
        /// <param name="order">The order.</param>
        /// <param name="discountOrderLines">The discount order lines.</param>
        internal static void AddDiscountOrderLinesToCache(Order order, OrderLineCollection discountOrderLines)
        {
            var cacheKey = GetDiscountOrderLinesCacheKey(order);
            Context.Current.Session[cacheKey] = discountOrderLines;
        }

        /// <summary>
        /// Clears the cache.
        /// </summary>
        /// <param name="order">The order.</param>
        internal static void ClearCache(Order order)
        {
            RemoveDiscountOrderLinesFromCache(order);
            ClearTranslationCache();
        }

        /// <summary>
        /// Gets the first live sales discount.
        /// </summary>
        /// <returns>SalesDiscount.</returns>
        internal static SalesDiscount GetSalesDiscount()
        {
            return SalesDiscount.GetSalesDiscounts(Common.Context.Language).FirstOrDefault(sd => !string.IsNullOrEmpty(sd.DiscountType) && string.Compare(sd.DiscountType, typeof(LiveSalesDiscountProvider).FullName, StringComparison.OrdinalIgnoreCase) == 0);
        }

        /// <summary>
        /// Removes discount order lines from cache.
        /// </summary>
        /// <param name="order">The order.</param>
        internal static void RemoveDiscountOrderLinesFromCache(Order order)
        {
            var cacheKey = GetDiscountOrderLinesCacheKey(order);
            Context.Current.Session.Remove(cacheKey);
        }

        /// <summary>
        /// Gets the discount order lines cache key
        /// </summary>
        /// <param name="order">Order used for generating the cache key</param>
        /// <returns>Cache key</returns>
        private static string GetDiscountOrderLinesCacheKey(Order order)
        {
            return "LiveIntegrationDiscounts" + order.AutoId;
        }

        /// <summary>
        /// Gets discount order lines from cache
        /// </summary>
        /// <param name="order">Order to get discount order lines for</param>
        /// <returns>A collection of discount order lines</returns>
        private static OrderLineCollection GetDiscountOrderLinesFromCache(Order order)
        {
            var cacheKey = GetDiscountOrderLinesCacheKey(order);

            var orderLines = Context.Current.Session[cacheKey] as OrderLineCollection;

            return orderLines;
        }

        /// <summary>
        /// Gets the discount name
        /// </summary>
        /// <param name="orderLineNode">order line node</param>
        /// <param name="orderLine">order line</param>
        /// <returns>Translated discount name</returns>
        internal static string GetDiscountName(XmlNode orderLineNode, OrderLine orderLine)
        {
            string discountName = orderLineNode?.SelectSingleNode("column [@columnName='OrderLineProductName']")?.InnerText;
            string translatedDiscountName = null;
            if (orderLine.OrderLineType == OrderLineType.ProductDiscount)
            {
                translatedDiscountName = Helpers.GetTranslation(Constants.OrderConfiguration.ProductDiscountText);
                discountName = string.IsNullOrEmpty(translatedDiscountName) ? Settings.Instance.ProductDiscountText : translatedDiscountName;
            }
            else if (orderLine.OrderLineType == OrderLineType.Discount)
            {
                translatedDiscountName = Helpers.GetTranslation(Constants.OrderConfiguration.OrderDiscountText);
                discountName = string.IsNullOrEmpty(translatedDiscountName) ? Settings.Instance.OrderDiscountText : translatedDiscountName;
            }
            if (!string.IsNullOrEmpty(translatedDiscountName))
            {
                if (Context.Current.Session[DiscountTranslationCultureSessionKey] == null)
                {
                    System.Globalization.CultureInfo culture = ExecutingContext.GetCulture(true);
                    Context.Current.Session[DiscountTranslationCultureSessionKey] = culture?.Name;
                }
            }
            return discountName;
        }

        /// <summary>
        /// Checks if tranlsation for discount order lines is correct
        /// </summary>
        /// <param name="order"></param>
        internal static void UpdateDiscountOrderLinesTranslation(Order order)
        {
            if (order != null && Context.Current.Session[DiscountTranslationCultureSessionKey] != null)
            {
                Diagnostics.ExecutionTable.Current.Add("LiveIntegration.LiveSalesDiscountProvider.UpdateDiscountOrderLinesTranslation START");
                System.Globalization.CultureInfo currentCulture = ExecutingContext.GetCulture(true);
                if (currentCulture != null)
                {
                    string cultureName = Context.Current.Session[DiscountTranslationCultureSessionKey].ToString();
                    if (!string.Equals(currentCulture.Name, cultureName, StringComparison.OrdinalIgnoreCase))
                    {
                        ClearTranslationCache();
                        foreach (OrderLine orderLine in order.OrderLines.Where(ol => ol.OrderLineType == OrderLineType.Discount || 
                            ol.OrderLineType == OrderLineType.ProductDiscount))
                        {
                            orderLine.ProductName = GetDiscountName(null, orderLine);
                        }
                    }
                }
                Diagnostics.ExecutionTable.Current.Add("LiveIntegration.LiveSalesDiscountProvider.UpdateDiscountOrderLinesTranslation END");
            }
        }

        private static void ClearTranslationCache()
        {
            if (Context.Current.Session[DiscountTranslationCultureSessionKey] != null)
            {
                Context.Current.Session.Remove(DiscountTranslationCultureSessionKey);
            }
        }
    }
}