﻿using Dynamicweb.Controls.Extensibility;
using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.LiveIntegration.XmlGenerators;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Extensibility.AddIns;
using System;
using System.IO;

namespace Dynamicweb.Ecommerce.LiveIntegration.RibbonBars
{
    /// <summary>
    /// Adds Export XML buttons to the Ribbon bar for order details.
    /// </summary>
    /// <seealso cref="RibbonBarBase" />
    [AddInName("Create XML")]
    [AddInDescription("Creates XML for an order")]
    [AddInTarget(RibbonBarAddInTarget.eCom.OrderEdit)]
    public class CreateXmlForOrderRibbonBar : RibbonBarBase
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="CreateXmlForOrderRibbonBar" /> class.
        /// </summary>
        /// <param name="ribbon">The ribbon to which the controls are added.</param>
        public CreateXmlForOrderRibbonBar(Controls.RibbonBar ribbon) : base(ribbon)
        {
        }

        /// <summary>
        /// Gets a value indicating whether [save order XML].
        /// </summary>
        /// <value><c>true</c> if [save order XML]; otherwise, <c>false</c>.</value>
        private static bool SaveOrderXml => Settings.Instance.SaveCopyOfOrderXml;

        /// <summary>
        /// Fires when this ribbon bar add in loads. Adds a download button to the Orders section.
        /// </summary>
        public override void Load()
        {
            var order = Ribbon.DataContext.DataSource as Order;

            if (order == null)
            {
                return;
            }

            var group = CreateLiveIntegrationRibbon("Orders");

            using (var buttonOriginalXml = CreateExportToXmlButton(group, "Original XML"))
            {
                var enableButton = SaveOrderXml && File.Exists(BuildXmlFileName(order));
                buttonOriginalXml.Disabled = !enableButton;
                if (enableButton)
                {
                    buttonOriginalXml.Title = "Downloads the original XML for an order as sent to the ERP";
                    buttonOriginalXml.EnableServerClick = true;
                    buttonOriginalXml.Click += GetOriginalXml;
                }
                else
                {
                    if (!SaveOrderXml)
                    {
                        buttonOriginalXml.Title = "This option is not available because saving XML files is not enabled in the Live Integration setup.";
                    }
                    else
                    {
                        buttonOriginalXml.Title = "This option is not available because the XML file does not exist.";
                    }

                    buttonOriginalXml.EnableServerClick = false;
                }

                using (var buttonCurrentXml = CreateExportToXmlButton(group, "Current XML"))
                {
                    buttonCurrentXml.Title = "Exports the order to an XML document";
                    buttonCurrentXml.EnableServerClick = true;
                    buttonCurrentXml.Click += GetCurrentXml;
                }
            }
        }

        /// <summary>
        /// Builds the name of the XML file.
        /// </summary>
        /// <param name="order">The order.</param>
        /// <returns>System.String.</returns>
        private static string BuildXmlFileName(Order order)
        {
            return OrderHandler.BuildXmlCopyPath(order.Id, OrderHandler.GetLogFolderForXmlCopies(order.CompletedDate));
        }

        /// <summary>
        /// Occurs when the button was clicked from edit order page. Gets the XML for the order and returns it to the browser.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void GetCurrentXml(object sender, EventArgs e)
        {
            var order = (Order)Ribbon.DataContext.DataSource;
            var xml = new OrderXmlGenerator().GenerateOrderXml(order, new OrderXmlGeneratorSettings { AddOrderFieldsToRequest = true, AddOrderLineFieldsToRequest = true, CreateOrder = true, Beautify = true, LiveIntegrationSubmitType = SubmitType.DownloadedFromBackEnd, ReferenceName = "OrdersPut" });
            StreamFile(xml, $"Order_{order.Id}.xml");
        }

        /// <summary>
        /// Gets the original XML by reading the original file from disk.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="EventArgs"/> instance containing the event data.</param>
        private void GetOriginalXml(object sender, EventArgs e)
        {
            var order = (Order)Ribbon.DataContext.DataSource;
            var xml = File.ReadAllText(BuildXmlFileName(order));
            StreamFile(xml, $"Order_{order.Id}.xml");
        }
    }
}