using Dynamicweb.Ecommerce.LiveIntegration.Configuration;
using Dynamicweb.Ecommerce.LiveIntegration.Extensions;
using Dynamicweb.Ecommerce.LiveIntegration.Products;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Environment;
using Dynamicweb.Extensibility.Notifications;
using Dynamicweb.Security.UserManagement;
using System;
using System.Linq;
using System.Xml;

namespace Dynamicweb.Ecommerce.LiveIntegration.XmlGenerators
{
    /// <summary>
    /// Generates XML for orders.
    /// </summary>
    /// <seealso cref="XmlGenerator" />
    internal class OrderXmlGenerator : XmlGenerator
    {
        /// <summary>
        /// Generates the order XML.
        /// </summary>
        /// <param name="order">The order.</param>
        /// <param name="settings">The settings.</param>
        /// <returns>System.String.</returns>
        internal string GenerateOrderXml(Order order, OrderXmlGeneratorSettings settings)
        {
            NotificationManager.Notify(Notifications.Order.OnBeforeGenerateOrderXml, new Notifications.Order.OnBeforeGenerateOrderXmlArgs(order, settings));
            var xmlDocument = BuildXmlDocument();
            var tablesNode = CreateAndAppendTablesNode(xmlDocument, settings);
            tablesNode.AppendChild(BuildOrderXml(xmlDocument, order, settings));
            tablesNode.AppendChild(BuildOrderLinesXml(xmlDocument, order, settings));
            if (settings.AddOrderLineFieldsToRequest)
            {
                tablesNode.AppendChild(BuildOrderLineFieldsXml(xmlDocument, order));
            }

            NotificationManager.Notify(Notifications.Order.OnAfterGenerateOrderXml, new Notifications.Order.OnAfterGenerateOrderXmlArgs(order, settings, xmlDocument));

            if (settings.Beautify)
            {
                return xmlDocument.Beautify();
            }

            return xmlDocument.InnerXml;
        }

        /// <summary>
        /// Adds the customer information to the XML node.
        /// </summary>
        /// <param name="orderNode">The order node.</param>
        /// <param name="order">The order.</param>
        private static void AddCustomerInformation(XmlElement orderNode, Order order)
        {
            var user = User.GetUserByID(order.CustomerAccessUserId) ?? (!ExecutingContext.IsBackEnd() ? User.GetCurrentExtranetUser() : null);

            AddChildXmlNode(orderNode, "OrderCustomerAccessUserExternalId", !string.IsNullOrWhiteSpace(user?.ExternalID) ? user.ExternalID : Settings.Instance.AnonymousUserKey);
            AddChildXmlNode(orderNode, "OrderCustomerNumber", !string.IsNullOrWhiteSpace(user?.CustomerNumber) ? user.CustomerNumber : Settings.Instance.AnonymousUserKey);
            AddChildXmlNode(orderNode, "OrderCustomerName", order.CustomerName);
            AddChildXmlNode(orderNode, "OrderCustomerAddress", order.CustomerAddress);
            AddChildXmlNode(orderNode, "OrderCustomerAddress2", order.CustomerAddress2);
            AddChildXmlNode(orderNode, "OrderCustomerCity", order.CustomerCity);
            AddChildXmlNode(orderNode, "OrderCustomerState", order.CustomerRegion);
            AddChildXmlNode(orderNode, "OrderCustomerZip", order.CustomerZip);
            AddChildXmlNode(orderNode, "OrderCustomerCountryCode", order.CustomerCountryCode);
            AddChildXmlNode(orderNode, "OrderCustomerEmail", order.CustomerEmail);
            AddChildXmlNode(orderNode, "OrderCustomerPhone", order.CustomerPhone);
            AddChildXmlNode(orderNode, "OrderCustomerFax", order.CustomerFax);
            AddChildXmlNode(orderNode, "OrderCustomerCompany", order.CustomerCompany);
            AddChildXmlNode(orderNode, "OrderCustomerComment", order.CustomerComment);
            AddChildXmlNode(orderNode, "OrderCustomerFirstName", order.CustomerFirstName);
            AddChildXmlNode(orderNode, "OrderCustomerSurname", order.CustomerSurname);
        }

        /// <summary>
        /// Adds the order delivery information to the XML node.
        /// </summary>
        /// <param name="orderNode">The order node.</param>
        /// <param name="order">The order.</param>
        private void AddOrderDeliveryInformation(XmlElement orderNode, Order order)
        {
            AddChildXmlNode(orderNode, "OrderDeliveryName", !string.IsNullOrWhiteSpace(order.DeliveryName) ? order.DeliveryName : order.CustomerName);
            AddChildXmlNode(orderNode, "OrderDeliveryAddress", order.DeliveryAddress);
            AddChildXmlNode(orderNode, "OrderDeliveryAddress2", order.DeliveryAddress2);
            AddChildXmlNode(orderNode, "OrderDeliveryCity", order.DeliveryCity);
            AddChildXmlNode(orderNode, "OrderDeliveryState", order.DeliveryRegion);
            AddChildXmlNode(orderNode, "OrderDeliveryZip", order.DeliveryZip);
            AddChildXmlNode(orderNode, "OrderDeliveryCountryCode", order.DeliveryCountryCode);
            AddChildXmlNode(orderNode, "OrderDeliveryEmail", order.DeliveryEmail);
            AddChildXmlNode(orderNode, "OrderDeliveryPhone", order.DeliveryPhone);
            AddChildXmlNode(orderNode, "OrderDeliveryFax", order.DeliveryFax);
            AddChildXmlNode(orderNode, "OrderDeliveryCompany", order.DeliveryCompany);
        }

        /// <summary>
        /// Appends the order fields to the XML node.
        /// </summary>
        /// <param name="orderNode">The order node.</param>
        /// <param name="order">The order.</param>
        private void AppendOrderFields(XmlElement orderNode, Order order)
        {
            foreach (var field in order.OrderFieldValues)
            {
                string value = field.Value?.ToIntegrationString() ?? string.Empty;
                AddChildXmlNode(orderNode, field.OrderField.SystemName, value, isCustomField: true);
            }
        }

        /// <summary>
        /// Builds the order line fields XML.
        /// </summary>
        /// <param name="xmlDocument">The XML document.</param>
        /// <param name="order">The order.</param>
        /// <returns>XmlNode.</returns>
        private XmlNode BuildOrderLineFieldsXml(XmlDocument xmlDocument, Order order)
        {
            var orderLineFields = CreateTableNode(xmlDocument, "EcomOrderLineFields");

            foreach (var orderLine in order.OrderLines.Where(x => x.OrderLineFieldValues.Any()))
            {
                var itemNode = CreateAndAppendItemNode(orderLineFields, "EcomOrderLineFields");
                foreach (var field in orderLine.OrderLineFieldValues)
                {
                    AddChildXmlNode(itemNode, "OrderLineFieldOrderLineId", orderLine.Id);
                    AddChildXmlNode(itemNode, "OrderLineFieldName", field.OrderLineFieldName ?? string.Empty);
                    AddChildXmlNode(itemNode, "OrderLineFieldValue", field.Value ?? string.Empty);
                    AddChildXmlNode(itemNode, "OrderLineFieldSystemName", field.OrderLineFieldSystemName ?? string.Empty);
                }
            }

            return orderLineFields;
        }

        /// <summary>
        /// Builds the order lines XML.
        /// </summary>
        /// <param name="xmlDocument">The XML document.</param>
        /// <param name="order">The order.</param>
        /// <param name="settings">Settings for generating orderline xml</param>
        /// <returns>Xml generated for an orderline</returns>
        private XmlNode BuildOrderLinesXml(XmlDocument xmlDocument, Order order, OrderXmlGeneratorSettings settings)
        {
            var tableNode = CreateTableNode(xmlDocument, "EcomOrderLines");

            // Order lines (products, taxes)
            foreach (var orderLine in order.OrderLines.Where(ol => !ol.IsDiscount()))
            {
                CreateOrderLineXml(tableNode, orderLine, settings);
            }

            // Order lines (order discounts, and product discounts)
            foreach (var orderLine in order.OrderLines.Where(ol => ol.IsDiscount()))
            {
                CreateOrderLineXml(tableNode, orderLine, settings);
            }

            return tableNode;
        }

        /// <summary>
        /// Builds the order XML.
        /// </summary>
        /// <param name="xmlDocument">The XML document.</param>
        /// <param name="order">The order.</param>
        /// <param name="settings">The settings.</param>
        /// <returns>XmlNode.</returns>
        private XmlNode BuildOrderXml(XmlDocument xmlDocument, Order order, OrderXmlGeneratorSettings settings)
        {
            var tableNode = CreateTableNode(xmlDocument, "EcomOrders");
            var itemNode = CreateAndAppendItemNode(tableNode, "EcomOrders");

            AddCustomerInformation(itemNode, order);
            AddOrderDeliveryInformation(itemNode, order);

            // do not use order.Modified in XML unless the field can be ignored for hash calculation
            AddChildXmlNode(itemNode, "CreateOrder", settings.CreateOrder.ToString());
            AddChildXmlNode(itemNode, "OrderId", order.Id);
            AddChildXmlNode(itemNode, "OrderAutoId", order.AutoId.ToString());
            AddChildXmlNode(itemNode, "OrderIntegrationOrderId", order.IntegrationOrderId);
            AddChildXmlNode(itemNode, "OrderCurrencyCode", order.CurrencyCode);
            AddChildXmlNode(itemNode, "OrderDate", order.Date.ToIntegrationString());
            AddChildXmlNode(itemNode, "OrderPaymentMethodName", order.PaymentMethod, true);
            AddChildXmlNode(itemNode, "OrderPaymentMethodId", order.PaymentMethodId);

            if (settings.ErpControlsShipping)
            {
                // AX handles shipping
                AddChildXmlNode(itemNode, "OrderShippingMethodName", string.Empty, true);
                AddChildXmlNode(itemNode, "OrderShippingMethodId", string.Empty);
                AddChildXmlNode(itemNode, "OrderShippingFee", string.Empty);
            }
            else
            {
                // Dynamicweb handles shipping
                AddChildXmlNode(itemNode, "OrderShippingMethodName", order.ShippingMethod, true);
                AddChildXmlNode(itemNode, "OrderShippingMethodId", order.ShippingMethodId);
                AddChildXmlNode(itemNode, "OrderShippingFee", order.ShippingFee.PriceWithVAT.ToIntegrationString());
                AddChildXmlNode(itemNode, "OrderShippingFeeWithoutVat", order.ShippingFee.PriceWithoutVAT.ToIntegrationString());
                AddChildXmlNode(itemNode, "OrderShippingItemKey", settings.ErpShippingItemKey);
            }

            AddChildXmlNode(itemNode, "OrderPriceTotal", order.TotalPrice.ToIntegrationString());
            AddChildXmlNode(itemNode, "OrderCaptureAmount", order.CaptureAmount.ToIntegrationString());
            AddChildXmlNode(itemNode, "OrderVoucherCode", order.VoucherCode);
            AddChildXmlNode(itemNode, "OrderTransactionId", order.TransactionNumber);
            AddChildXmlNode(itemNode, "OrderStateId", order.StateId);
            AddChildXmlNode(itemNode, "OrderStateName", order.OrderState.Name, true);
            AddChildXmlNode(itemNode, "ErpControlsDiscount", settings.ErpControlsDiscount.ToString());

            if (settings.AddOrderFieldsToRequest)
            {
                AppendOrderFields(itemNode, order);
            }

            return tableNode;
        }

        /// <summary>
        /// Creates the order line XML.
        /// </summary>
        /// <param name="orderLinesNode">The order lines node.</param>
        /// <param name="orderline">The orderline.</param>
        /// <param name="settings">Settings for generating the orderline xml.</param>
        private void CreateOrderLineXml(XmlNode orderLinesNode, OrderLine orderline, OrderXmlGeneratorSettings settings)
        {
            NotificationManager.Notify(Notifications.OrderLine.OnBeforeGenerateOrderLineXml, 
                new Notifications.OrderLine.OnBeforeGenerateOrderLineXmlArgs(orderline, settings));

            if (orderline.Bom && !Settings.Instance.AddOrderLinePartsToRequest)
            {
                return;
            }

            XmlElement itemNode = CreateAndAppendItemNode(orderLinesNode, "EcomOrderLines");

            // do not use ol.Date in XML unless the field can be ignored for hash calculation
            // do not use ol.Modified in XML unless the field can be ignored for hash calculation
            AddChildXmlNode(itemNode, "OrderLineId", orderline.Id);
            AddChildXmlNode(itemNode, "OrderLineOrderId", orderline.OrderId);
            AddChildXmlNode(itemNode, "OrderLineParentLineId", orderline.ParentLineId);
            AddChildXmlNode(itemNode, "OrderLineProductId", 
                settings.CalculateOrderUsingProductNumber ? orderline.ProductNumber : orderline.ProductId);
            AddChildXmlNode(itemNode, "OrderLineProductVariantId", orderline.ProductVariantId);
            AddChildXmlNode(itemNode, "OrderLineProductNumber", orderline.ProductNumber);
            AddChildXmlNode(itemNode, "OrderLineProductName", orderline.ProductName); // Product name or discount description
            AddChildXmlNode(itemNode, "OrderLineProductIdentifier", ProductManager.ProductProvider.GetProductIdentifier(orderline.Product));
            AddChildXmlNode(itemNode, "OrderLineQuantity", orderline.Quantity.ToIntegrationString());
            AddChildXmlNode(itemNode, "OrderLineUnitId", orderline.UnitId ?? string.Empty);
            AddChildXmlNode(itemNode, "OrderLinePriceWithoutVat", orderline.Price.PriceWithoutVAT.ToIntegrationString());
            AddChildXmlNode(itemNode, "OrderLineUnitPriceWithoutVat", orderline.UnitPrice.PriceWithoutVAT.ToIntegrationString());
            AddChildXmlNode(itemNode, "NetPrice", orderline.IsProduct() ? orderline.Product.DefaultPrice.ToIntegrationString() : string.Empty);
            AddChildXmlNode(itemNode, "OrderLineType", ((int)orderline.OrderLineType).ToString());
            AddChildXmlNode(itemNode, "OrderLineTypeName", orderline.OrderLineType.ToString());
            AddChildXmlNode(itemNode, "OrderLineBom", orderline.Bom.ToString());
            AddChildXmlNode(itemNode, "OrderLineBomItemId", orderline.BomItemId);
            AddChildXmlNode(itemNode, "OrderLineGiftCardCode", orderline.GiftCardCode);
            AddChildXmlNode(itemNode, "OrderLineIsGiftCardDiscount", (!string.IsNullOrEmpty(orderline.GiftCardCode)).ToString(), isCustomField: true);
            AddChildXmlNode(itemNode, "OrderLineFieldValues", orderline.OrderLineFieldValues.ToXml().InnerXml);
            if (!settings.ErpControlsDiscount && orderline.IsDiscount())
            {
                AddChildXmlNode(itemNode, "OrderLineDiscountId", orderline.DiscountId);                
                var discount = Orders.Discounts.Discount.GetDiscountById(Core.Converter.ToInt64(orderline.DiscountId));
                if(discount != null)
                {
                    AddChildXmlNode(itemNode, "OrderLineDiscountName", discount.Name);
                    AddChildXmlNode(itemNode, "OrderLineCampaignName", discount.CampaignName);
                    AddChildXmlNode(itemNode, "OrderLineDiscountType", discount.DiscountType.ToString());                    
                    AddChildXmlNode(itemNode, "OrderLineDiscountValue", Helpers.GetDiscountValue(discount, orderline));                    
                }
            }

            if (Settings.Instance.AddOrderLinePartsToRequest)
            {
                foreach (var kitProductOrderLine in orderline.BomOrderLines)
                {
                    CreateOrderLineXml(orderLinesNode, kitProductOrderLine, settings);
                }
            }

            NotificationManager.Notify(Notifications.OrderLine.OnAfterGenerateOrderLineXml, new Notifications.OrderLine.OnAfterGenerateOrderLineXmlArgs(orderline, itemNode));
        }
    }
}